/**
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __INFERENCE_ENGINE_IMPL_OPENCV_H__
#define __INFERENCE_ENGINE_IMPL_OPENCV_H__

#include <inference_engine_vision.h>

#include <opencv2/dnn.hpp>
#include <opencv2/imgproc.hpp>

#include <dlog/dlog.h>
/**
 * @file inference_engine_opencv_private.h
 * @brief This file contains the InferenceOpenCV class which
 *        provide OpenCV based inference functionality
 */

#ifdef LOG_TAG
#undef LOG_TAG
#endif

#define LOG_TAG "INFERENCE_ENGINE_OPENCV"

using namespace InferenceEngineInterface::Vision;
using namespace InferenceEngineInterface::Common;

namespace InferenceEngineImpl {
namespace OpenCVImpl {

class InferenceOpenCV : public IInferenceEngineVision {
public:
    InferenceOpenCV(std::string protoFile,
                    std::string weightFile,
                    std::string userFile);

    ~InferenceOpenCV();

    // Input Tensor Params

    int SetInputTensorParam() override;

    int SetInputTensorParamNode(std::string node = "input") override;

    int SetInputTensorParamInput(int width, int height, int dim, int ch) override;

    int SetInputTensorParamNorm(double deviation = 1.0, double mean = 0.0) override;

    // Output Tensor Params
    int SetOutputTensorParam() override;

    int SetOutputTensorParamThresHold(double threshold) override;

    int SetOutputTensorParamNumbers(int number) override;

    int SetOutputTensorParamType(int type) override;

    int SetOutputTensorParamNodes(std::vector<std::string> nodes) override;

    int SetTargetDevice(inference_target_type_e type) override;

    int Load() override;

    int CreateInputLayerPassage() override;

    int PrepareInputLayerPassage() override;

    int PrepareInputLayerPassage(inference_input_type_e type) override;

    int Run(cv::Mat tensor) override;

    int Run(std::vector<float> tensor) override;

    int GetInferenceResult(ImageClassificationResults& results) override;

    int GetInferenceResult(ObjectDetectionResults& results) override;

    int GetInferenceResult(FaceDetectionResults& results) override;

    int GetInferenceResult(FacialLandMarkDetectionResults& results) override;

    int GetInferenceResult(std::vector<std::vector<int>>& dimInfo, std::vector<float*>& results);

    int GetNumberOfOutputs() override;

    void SetUserListName(std::string userList) override;

public:
    int SetUserFile();
    int setInput(cv::Mat cvImg);
    
private:
    cv::Mat mInputBlob;
    std::vector<cv::Mat> mOutputProb;
    cv::dnn::Net mNet; /**< Network associated with a network model */

    cv::String mInputLayer;
    std::vector<cv::String> mOutputLayer;
    int mCh;
    int mDim;
    cv::Size mInputSize; /**< input tensor size */

    double mDeviation;
    double mMean;
    double mThreshold;
    int mOutputNumbers;
    cv::Size mSourceSize; /**< input image's size */

    std::string mConfigFile;
    std::string mWeightFile;
    std::string mUserFile;
    std::vector<std::string> mUserListName;
};

} /* InferenceEngineImpl */
} /* TFImpl */

#endif /* __MEDIA_VISION_INFERENCE_OPENCV_H__ */
